@@ -8,28 +8,29 @@ use DynaLoader ();
 use vars qw($VERSION @ISA);
 use vars qw{$err $errstr $drh $sqlite_version};
 BEGIN {
-    $VERSION = '1.21';
+    $VERSION = '1.23';
     @ISA     = ('DynaLoader');
 
+    # Initialize errors
+    $err     = undef;
+    $errstr  = undef;
+
     # Driver singleton
-    $drh     = undef;
+    $drh = undef;
+
+    # sqlite_version cache
+    $sqlite_version = undef;
 }
 
 __PACKAGE__->bootstrap($VERSION);
 
 sub driver {
-    return $drh if $drh;
-    my ($class, $attr) = @_;
-
-    $class .= "::dr";
-
-    $drh = DBI::_new_drh($class, {
+    $drh or
+    $drh = DBI::_new_drh( "$_[0]::dr", {
         Name        => 'SQLite',
         Version     => $VERSION,
         Attribution => 'DBD::SQLite by Matt Sergeant et al',
-    });
-
-    return $drh;
+    } );
 }
 
 sub CLONE {
@@ -52,19 +53,40 @@ sub connect {
 
     my $real = $dbname;
     if ( $dbname =~ /=/ ) {
-        foreach my $attrib ( split(/;/, $dbname ) ) {
-            my ($k, $v) = split(/=/, $attrib, 2);
-            if ($k eq 'dbname') {
-                $real = $v;
+        foreach my $attrib ( split(/;/, $dbname) ) {
+            my ($key, $value) = split(/=/, $attrib, 2);
+            if ( $key eq 'dbname' ) {
+                $real = $value;
             } else {
-                # TODO: add to attribs
+                $attr->{$key} = $value;
             }
         }
     }
 
+    # To avoid unicode and long file name problems on Windows,
+    # convert to the shortname if the file (or parent directory) exists.
+    if ( $^O eq 'MSWin32' and $real ne ':memory:' ) {
+        require Win32;
+        require File::Basename;
+        my ($file, $dir, $suffix) = File::Basename::fileparse($real);
+        my $short = Win32::GetShortPathName($real);
+        if ( $short && -f $short ) {
+            # Existing files will work directly.
+            $real = $short;
+        } elsif ( -d $dir ) {
+            # We are creating a new file.
+            # Does the directory it's in at least exist?
+            $real = join '', grep { defined } Win32::GetShortPathName($dir), $file, $suffix;
+        } else {
+            # SQLite can't do mkpath anyway.
+            # So let it go through as it and fail.
+        }
+    }
+
+    # Hand off to the actual login function
     DBD::SQLite::db::_login($dbh, $real, $user, $auth) or return undef;
 
-    # install perl collations
+    # Install perl collations
     my $perl_collation        = sub { $_[0] cmp $_[1] };
     my $perl_locale_collation = sub { use locale; $_[0] cmp $_[1] };
     $dbh->func( "perl",       $perl_collation,        "create_collation" );
@@ -82,19 +104,20 @@ sub connect {
 package DBD::SQLite::db;
 
 sub prepare {
-    my ($dbh, $statement, @attribs) = @_;
+    my $dbh = shift;
+    my $sql = shift;
 
-    my $sth = DBI::_new_sth($dbh, {
-        Statement => $statement,
-    });
+    my $sth = DBI::_new_sth( $dbh, {
+        Statement => $sql,
+    } );
 
-    DBD::SQLite::st::_prepare($sth, $statement, @attribs) or return undef;
+    DBD::SQLite::st::_prepare($sth, $sql, @_) or return undef;
 
     return $sth;
 }
 
 sub _get_version {
-    return( DBD::SQLite::db::FETCH($_[0], 'sqlite_version') );
+    return ( DBD::SQLite::db::FETCH($_[0], 'sqlite_version') );
 }
 
 my %info = (
@@ -110,11 +133,11 @@ sub get_info {
     return $v;
 }
 
+# SQL/CLI (ISO/IEC JTC 1/SC 32 N 0595), 6.63 Tables
+# Based on DBD::Oracle's
+# See also http://www.ch-werner.de/sqliteodbc/html/sqliteodbc_8c.html#a117
 sub table_info {
     my ($dbh, $cat_val, $sch_val, $tbl_val, $typ_val) = @_;
-    # SQL/CLI (ISO/IEC JTC 1/SC 32 N 0595), 6.63 Tables
-    # Based on DBD::Oracle's
-    # See also http://www.ch-werner.de/sqliteodbc/html/sqliteodbc_8c.html#a117
 
     my @where = ();
     my $sql;
@@ -252,50 +275,46 @@ sub primary_key_info {
 }
 
 sub type_info_all {
-    my ($dbh) = @_;
     return; # XXX code just copied from DBD::Oracle, not yet thought about
-    my $names = {
-        TYPE_NAME          =>  0,
-        DATA_TYPE          =>  1,
-        COLUMN_SIZE        =>  2,
-        LITERAL_PREFIX     =>  3,
-        LITERAL_SUFFIX     =>  4,
-        CREATE_PARAMS      =>  5,
-        NULLABLE           =>  6,
-        CASE_SENSITIVE     =>  7,
-        SEARCHABLE         =>  8,
-        UNSIGNED_ATTRIBUTE =>  9,
-        FIXED_PREC_SCALE   => 10,
-        AUTO_UNIQUE_VALUE  => 11,
-        LOCAL_TYPE_NAME    => 12,
-        MINIMUM_SCALE      => 13,
-        MAXIMUM_SCALE      => 14,
-        SQL_DATA_TYPE      => 15,
-        SQL_DATETIME_SUB   => 16,
-        NUM_PREC_RADIX     => 17,
-    };
-    my $ti = [
-        $names,
-        [ 'CHAR', 1, 255, '\'', '\'', 'max length', 1, 1, 3,
-            undef, '0', '0', undef, undef, undef, 1, undef, undef
-        ],
-        [ 'NUMBER', 3, 38, undef, undef, 'precision,scale', 1, '0', 3,
-            '0', '0', '0', undef, '0', 38, 3, undef, 10
-        ],
-        [ 'DOUBLE', 8, 15, undef, undef, undef, 1, '0', 3,
-            '0', '0', '0', undef, undef, undef, 8, undef, 10
-        ],
-        [ 'DATE', 9, 19, '\'', '\'', undef, 1, '0', 3,
-            undef, '0', '0', undef, '0', '0', 11, undef, undef
-        ],
-        [ 'VARCHAR', 12, 1024*1024, '\'', '\'', 'max length', 1, 1, 3,
-            undef, '0', '0', undef, undef, undef, 12, undef, undef
-        ]
-    ];
-    return $ti;
+#    return [
+#        {
+#            TYPE_NAME          =>  0,
+#            DATA_TYPE          =>  1,
+#            COLUMN_SIZE        =>  2,
+#            LITERAL_PREFIX     =>  3,
+#            LITERAL_SUFFIX     =>  4,
+#            CREATE_PARAMS      =>  5,
+#            NULLABLE           =>  6,
+#            CASE_SENSITIVE     =>  7,
+#            SEARCHABLE         =>  8,
+#            UNSIGNED_ATTRIBUTE =>  9,
+#            FIXED_PREC_SCALE   => 10,
+#            AUTO_UNIQUE_VALUE  => 11,
+#            LOCAL_TYPE_NAME    => 12,
+#            MINIMUM_SCALE      => 13,
+#            MAXIMUM_SCALE      => 14,
+#            SQL_DATA_TYPE      => 15,
+#            SQL_DATETIME_SUB   => 16,
+#            NUM_PREC_RADIX     => 17,
+#        },
+#        [ 'CHAR', 1, 255, '\'', '\'', 'max length', 1, 1, 3,
+#            undef, '0', '0', undef, undef, undef, 1, undef, undef
+#        ],
+#        [ 'NUMBER', 3, 38, undef, undef, 'precision,scale', 1, '0', 3,
+#            '0', '0', '0', undef, '0', 38, 3, undef, 10
+#        ],
+#        [ 'DOUBLE', 8, 15, undef, undef, undef, 1, '0', 3,
+#            '0', '0', '0', undef, undef, undef, 8, undef, 10
+#        ],
+#        [ 'DATE', 9, 19, '\'', '\'', undef, 1, '0', 3,
+#            undef, '0', '0', undef, '0', '0', 11, undef, undef
+#        ],
+#        [ 'VARCHAR', 12, 1024*1024, '\'', '\'', 'max length', 1, 1, 3,
+#            undef, '0', '0', undef, undef, undef, 12, undef, undef
+#        ]
+#    ];
 }
 
-
 # Taken from Fey::Loader::SQLite
 sub column_info {
     my($dbh, $catalog, $schema, $table, $column) = @_;
@@ -373,7 +392,7 @@ __END__
 
 =head1 NAME
 
-DBD::SQLite - Self Contained RDBMS in a DBI Driver
+DBD::SQLite - Self-contained RDBMS in a DBI Driver
 
 =head1 SYNOPSIS
 
@@ -383,11 +402,11 @@ DBD::SQLite - Self Contained RDBMS in a DBI Driver
 =head1 DESCRIPTION
 
 SQLite is a public domain RDBMS database engine that you can find
-at L<http://www.hwaci.com/sw/sqlite/>.
+at L<http://www.sqlite.org/>.
 
 Rather than ask you to install SQLite first, because SQLite is public
-domain, DBD::SQLite includes the entire thing in the distribution. So
-in order to get a fast transaction capable RDBMS working for your
+domain, B<DBD::SQLite> includes the entire thing in the distribution.
+So in order to get a fast transaction capable RDBMS working for your
 perl project you simply have to install this module, and B<nothing>
 else.
 
@@ -397,16 +416,16 @@ SQLite supports the following features:
 
 =item Implements a large subset of SQL92
 
-See L<http://www.hwaci.com/sw/sqlite/lang.html> for details.
+See L<http://www.sqlite.org/lang.html> for details.
 
 =item A complete DB in a single disk file
 
 Everything for your database is stored in a single disk file, making it
-easier to move things around than with DBD::CSV.
+easier to move things around than with L<DBD::CSV>.
 
 =item Atomic commit and rollback
 
-Yes, DBD::SQLite is small and light, but it supports full transactions!
+Yes, B<DBD::SQLite> is small and light, but it supports full transactions!
 
 =item Extensible
 
@@ -435,16 +454,16 @@ limited by the typeless nature of the SQLite database.
 
 =item sqlite_version
 
-Returns the version of the SQLite library which DBD::SQLite is using,
+Returns the version of the SQLite library which B<DBD::SQLite> is using,
 e.g., "2.8.0". Can only be read.
 
 =item unicode
 
-If set to a true value, DBD::SQLite will turn the UTF-8 flag on for all text
-strings coming out of the database. For more details on the UTF-8 flag see
+If set to a true value, B<DBD::SQLite> will turn the UTF-8 flag on for all text
+strings coming out of the database (this feature is currently disabled for perl < 5.8.5). For more details on the UTF-8 flag see
 L<perlunicode>. The default is for the UTF-8 flag to be turned off.
 
-Also note that due to some bizareness in SQLite's type system (see
+Also note that due to some bizarreness in SQLite's type system (see
 L<http://www.sqlite.org/datatype3.html>), if you want to retain
 blob-style behavior for B<some> columns under C<< $dbh->{unicode} = 1
 >> (say, to store images in the database), you have to state so
@@ -470,7 +489,7 @@ This method returns the last inserted rowid. If you specify an INTEGER PRIMARY
 KEY as the first column in your table, that is the column that is returned.
 Otherwise, it is the hidden ROWID column. See the sqlite docs for details.
 
-Note: You can now use $dbh->last_insert_id() if you have a recent version of
+Note: You can now use $dbh-E<gt>last_insert_id() if you have a recent version of
 DBI.
 
 =head2 $dbh->func('busy_timeout')
@@ -528,16 +547,41 @@ be used from SQL.
 =item $code_ref
 
 This should be a reference to the function's implementation.
+The driver will check that this is a proper sorting function.
 
 =back
 
-By default, the collations "perl" and "perllocale" are created for you.
+Collations C<binary> and C<nocase> are builtin within SQLite.
+Collations C<perl> and C<perllocale> are builtin within 
+the B<DBD::SQLite> driver, and correspond to the 
+Perl C<cmp> operator with or without the L<locale> pragma; 
+so you can write for example 
 
-These allow sorting in Perl terms using "cmp", in both locale and non-locale
-forms. For example, the following does a locale-aware Perl cmp sort.
+  CREATE TABLE foo(
+      txt1 COLLATE perl,
+      txt2 COLLATE perllocale,
+      txt3 COLLATE nocase
+  )
+
+or
 
   SELECT * FROM foo ORDER BY name COLLATE perllocale
 
+If the attribute C<< $dbh->{unicode} >> is set, strings coming from
+the database and passed to the collation function will be properly
+tagged with the utf8 flag; but this only works if the 
+C<unicode> attribute is set B<before> the call to 
+C<create_collation>. The recommended way to activate unicode
+is to set the parameter at connection time :
+
+  my $dbh = DBI->connect(
+      "dbi:SQLite:dbname=foo", "", "", 
+      {
+          RaiseError => 1,
+          unicode    => 1,
+      }
+  );
+
 =head2 $dbh->func( $name, $argc, $pkg, 'create_aggregate' )
 
 This method will register a new aggregate function which can then be used
@@ -630,6 +674,32 @@ The aggregate function can then be used as:
   FROM results
   GROUP BY group_name;
 
+=head2 $dbh->func( $n_opcodes, $code_ref, 'progress_handler' )
+
+This method registers a handler to be invoked 
+periodically during long running calls to SQLite.
+An example use for this interface is to keep a GUI 
+updated during a large query.
+The parameters are:
+
+=over
+
+=item $n_opcodes
+
+The progress handler is invoked once for every C<$n_opcodes>
+virtual machine opcodes in SQLite.
+
+=item $handler
+
+Reference to the handler subroutine.  If the progress handler returns
+non-zero, the SQLite operation is interrupted. This feature can be used to
+implement a "Cancel" button on a GUI dialog box.
+
+Set this argument to C<undef> if you want to unregister a previous
+progress handler.
+
+=back
+
 =head1 BLOBS
 
 As of version 1.11, blobs should "just work" in SQLite as text columns. However
@@ -669,7 +739,7 @@ the DBI module. Just type:
 On the command line to access the file F<foo.db>.
 
 Alternatively you can install SQLite from the link above without conflicting
-with DBD::SQLite and use the supplied C<sqlite> command line tool.
+with B<DBD::SQLite> and use the supplied C<sqlite> command line tool.
 
 =head1 PERFORMANCE
 
@@ -715,26 +785,14 @@ There're several pended RT bugs/patches at the moment
 
 Here's the list.
 
-L<http://rt.cpan.org/Public/Bug/Display.html?id=41631>
-(patch required)
-
-L<http://rt.cpan.org/Public/Bug/Display.html?id=40594>
-(patch required, and the following tests may break)
-
-L<http://rt.cpan.org/Public/Bug/Display.html?id=30167>
-(need to see what is the best solution right now)
-
-L<http://rt.cpan.org/Public/Bug/Display.html?id=36836>
-(patch required)
-
-L<http://rt.cpan.org/Public/Bug/Display.html?id=13631>
-(test required)
-
 L<http://rt.cpan.org/Public/Bug/Display.html?id=35449>
-(break tests)
+(breaks tests)
 
 L<http://rt.cpan.org/Public/Bug/Display.html?id=29629>
-(patch required)
+(requires a patch)
+
+L<http://rt.cpan.org/Public/Bug/Display.html?id=29058>
+(requires a patch)
 
 Switch tests to L<Test::More> to support more advanced testing behaviours
 
